@@ -49,9 +49,17 @@ def expand_block(
         return
 
     log.debug("Expanding block %r", block)
-    # Filter out falsy blocks, i.e. None values.
-    # Allowing expanders to yield None makes them more consise.
-    yield from filter(None, expander(block))
+    for dependency in expander(block):
+        if not dependency:
+            # Filter out falsy blocks, i.e. None values.
+            # Allowing expanders to yield None makes them more consise.
+            continue
+        if dependency.code == b"DATA":
+            log.warn(
+                "expander yielded block %s which will be ignored in later iteration",
+                dependency,
+            )
+        yield dependency
 
 
 def dna_code(block_code: str):
@@ -85,15 +93,56 @@ def _expand_generic_nodetree(block: blendfile.BlendFileBlock):
     assert block.dna_type.dna_type_id == b"bNodeTree"
 
     nodes = block.get_pointer((b"nodes", b"first"))
+
+    # See DNA_node_types.h
+    socket_types_with_value_pointer = {
+        cdefs.SOCK_OBJECT,  #  bNodeSocketValueObject
+        cdefs.SOCK_IMAGE,  #  bNodeSocketValueImage
+        cdefs.SOCK_COLLECTION,  #  bNodeSocketValueCollection
+        cdefs.SOCK_TEXTURE,  #  bNodeSocketValueTexture
+        cdefs.SOCK_MATERIAL,  #  bNodeSocketValueMaterial
+    }
+
     for node in iterators.listbase(nodes):
         if node[b"type"] == cdefs.CMP_NODE_R_LAYERS:
             continue
-        yield node
 
         # The 'id' property points to whatever is used by the node
         # (like the image in an image texture node).
         yield node.get_pointer(b"id")
 
+        # Default values of inputs can also point to ID datablocks.
+        inputs = node.get_pointer((b"inputs", b"first"))
+        for input in iterators.listbase(inputs):
+            if input[b"type"] not in socket_types_with_value_pointer:
+                continue
+            value_container = input.get_pointer(b"default_value")
+            if not value_container:
+                continue
+            value = value_container.get_pointer(b"value")
+            yield value
+
+
+def _expand_generic_idprops(block: blendfile.BlendFileBlock):
+    """Yield ID datablocks and their libraries referenced from ID properties."""
+
+    # TODO(@sybren): this code is very crude, and happens to work on ID
+    # properties of Geometry Nodes modifiers, which is what it was written for.
+    # It should probably be rewritten to properly iterate over & recurse into
+    # all groups.
+    settings_props = block.get_pointer((b"settings", b"properties"))
+    if not settings_props:
+        return
+
+    subprops = settings_props.get_pointer((b"data", b"group", b"first"))
+    for idprop in iterators.listbase(subprops):
+        if idprop[b"type"] != cdefs.IDP_ID:
+            continue
+        id_datablock = idprop.get_pointer((b"data", b"pointer"))
+        if not id_datablock:
+            continue
+        yield id_datablock
+
 
 def _expand_generic_nodetree_id(block: blendfile.BlendFileBlock):
     block_ntree = block.get_pointer(b"nodetree", None)
@@ -243,6 +292,7 @@ def _expand_object(block: blendfile.BlendFileBlock):
         # Currently only node groups are supported. If the support should expand
         # to more types, something more intelligent than this should be made.
         if mod_type == cdefs.eModifierType_Nodes:
+            yield from _expand_generic_idprops(block_mod)
             yield block_mod.get_pointer(b"node_group")
 
 
